@@ -27,7 +27,7 @@ ClassBoxplot <- ClassBoxPlot <- function(Data, Cls, ColorSequence = DataVisualiz
     NoNanInd <- which(!is.nan(Data))
     Data <- Data[NoNanInd]
     Cls <- Cls[NoNanInd]
-    
+    Cls=checkCls(Cls,AnzData)
     #ClCou <- ClassCount(Cls)
     UniqueClasses = unique(Cls)#ClCou$UniqueClasses
     #CountPerClass = ClCou$countPerClass
@@ -63,7 +63,7 @@ ClassBoxplot <- ClassBoxPlot <- function(Data, Cls, ColorSequence = DataVisualiz
     
     if (!PlotLegend)
       ggobject <- ggobject + theme(legend.position = "none")
-    ggobject
-    return(list(ClassData = ClassData, ggobject = ggobject))
+    print(ggobject)
+    return(invisible(list(ClassData = ClassData, ggobject = ggobject)))
     
   }
\ No newline at end of file
